package com.interswitchng.isw_mobile_sdk

import android.app.Application
import android.content.Context
import androidx.annotation.NonNull
import com.interswitchng.iswmobilesdk.IswMobileSdk
import com.interswitchng.iswmobilesdk.shared.models.core.Environment
import com.interswitchng.iswmobilesdk.shared.models.core.IswPaymentInfo
import com.interswitchng.iswmobilesdk.shared.models.core.IswPaymentResult
import com.interswitchng.iswmobilesdk.shared.models.core.IswSdkConfig
import io.flutter.embedding.android.FlutterActivity
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import java.util.*

/**
 * IswMobileSdkPlugin
 */
class IswMobileSdkPlugin : MethodCallHandler,FlutterPlugin, ActivityAware {



    private lateinit var channel : MethodChannel

    private var application: Application? = null


    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPluginBinding) {
        channel = MethodChannel(
            flutterPluginBinding.binaryMessenger,
            "com.interswitchng.isw_mobile_sdk"
        )
        channel.setMethodCallHandler(this)
    }


    override fun onDetachedFromEngine(binding: FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }


    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "initialize" -> {
                val configMap = call.argument<Map<String, String>>("config")
                val env = call.argument<String>("env")
                assert(configMap != null)
                assert(env != null)
                application?.let { IswMobileSdk.initialize(it, getConfig(configMap, env)) }
                result.success(null)
            }
            "pay" -> {
                val infoMap = call.argument<Map<String, String>>("paymentInfo")
                IswMobileSdk.instance
                    .pay(getPaymentInfo(infoMap), IswSdkCallback(call, result))
            }
            else -> result.notImplemented()
        }
    }

    private fun getConfig(configMap: Map<String, String>?, env: String?): IswSdkConfig {
        // extract configurations
        val merchantId = configMap!!["merchantId"]
        val merchantSecrete = configMap["merchantSecrete"]
        val merchantCode = configMap["merchantCode"]
        val currencyCode = configMap["currencyCode"]
        val isProduction = env == Environment.PRODUCTION.name
        val isSandbox = env == Environment.SANDBOX.name

        // create configuration
        val config = IswSdkConfig(merchantId, merchantSecrete, merchantCode, currencyCode)

        // set environment
        if (isProduction) config.setEnv(Environment.PRODUCTION) else if (isSandbox) config.setEnv(
            Environment.SANDBOX
        )
        return config
    }

    private fun getPaymentInfo(infoMap: Map<String, String>?): IswPaymentInfo {
        val customerId = infoMap!!["customerId"]
        val customerName = infoMap["customerName"]
        val customerEmail = infoMap["customerEmail"]
        val customerMobile = infoMap["customerMobile"]
        val reference = infoMap["reference"]
        val amount = infoMap["amount"]


        return IswPaymentInfo(
            customerId.toString(),
            customerName.toString(),
            customerEmail.toString(),
            customerMobile.toString(),
            reference.toString(),
            amount.toString().toLong()
        )
    }

    internal class IswSdkCallback(var call: MethodCall, var result: MethodChannel.Result) :
        IswMobileSdk.IswPaymentCallback {
        override fun onUserCancel() {
            result.success(null)
        }

        override fun onPaymentCompleted(iswPaymentResul: IswPaymentResult) {
            result.success(getResult(iswPaymentResul))
        }

        private fun getResult(result: IswPaymentResult): Map<String, String> {
            val map: MutableMap<String, String> = HashMap()
            map["responseCode"] = result.responseCode
            map["responseDescription"] = result.responseDescription
            map["transactionReference"] = result.transactionReference
            map["amount"] = result.amount.toString() + ""
            map["isSuccessful"] = result.isSuccessful.toString() + ""
            map["channel"] = result.channel.name
            return map
        }
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        application = binding.activity.application

    }

    override fun onDetachedFromActivityForConfigChanges() {

    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {

    }

    override fun onDetachedFromActivity() {

    }

}